@@ -2,7 +2,6 @@
 '''HTTP protocol.'''
 import abc
 import collections
-import datetime
 import errno
 import gettext
 import logging
@@ -266,13 +265,15 @@ class Connection(object):
     def _connect(self):
         '''Connect the socket if not already connected.'''
         if self._connected:
+            self._io_stream.set_close_callback(self._stream_closed_callback)
             return
 
         yield self._make_socket()
 
         _logger.debug('Connecting to {0}.'.format(self._address))
         try:
-            yield self._io_stream.connect(self._address, self._host)
+            yield tornado.gen.Task(
+                self._io_stream.connect, self._address, self._host)
         except (ssl.SSLError, tornado.netutil.SSLCertificateError,
         SSLVerficationError) as error:
             raise SSLVerficationError('SSLError: {error}'.format(
@@ -462,13 +463,17 @@ class Connection(object):
             yield self._read_response_until_close(response)
             return
 
-        def response_callback(data):
+        data_queue = self._io_stream.read_bytes_queue(body_size)
+
+        while True:
+            data = yield data_queue.get()
+
+            if data is None:
+                break
+
             self._events.response_data.fire(data)
             response.body.content_file.write(self._decompress_data(data))
 
-        yield tornado.gen.Task(self._io_stream.read_bytes, body_size,
-            streaming_callback=response_callback)
-
         response.body.content_file.write(self._flush_decompressor())
 
     @tornado.gen.coroutine
@@ -486,8 +491,7 @@ class Connection(object):
     def _read_response_chunk(self, response):
         '''Read a single chunk of the chunked transfer encoding.'''
         _logger.debug('Reading chunk.')
-        chunk_size_hex = yield tornado.gen.Task(
-            self._io_stream.read_until_regex, b'[^\n\r]+')
+        chunk_size_hex = yield self._io_stream.read_until_regex(b'[^\n\r]+')
 
         self._events.response_data.fire(chunk_size_hex)
 
@@ -501,18 +505,21 @@ class Connection(object):
         if not chunk_size:
             raise tornado.gen.Return(chunk_size)
 
-        newline_data = yield tornado.gen.Task(
-            self._io_stream.read_until, b'\n')
+        newline_data = yield self._io_stream.read_until(b'\n')
 
         self._events.response_data.fire(newline_data)
 
-        def response_callback(data):
+        data_queue = self._io_stream.read_bytes_queue(chunk_size)
+
+        while True:
+            data = yield data_queue.get()
+
+            if data is None:
+                break
+
             self._events.response_data.fire(data)
             response.body.content_file.write(self._decompress_data(data))
 
-        yield tornado.gen.Task(self._io_stream.read_bytes, chunk_size,
-            streaming_callback=response_callback)
-
         response.body.content_file.write(self._flush_decompressor())
 
         raise tornado.gen.Return(chunk_size)
@@ -521,8 +528,7 @@ class Connection(object):
     def _read_response_chunked_trailer(self, response):
         '''Read the HTTP trailer fields.'''
         _logger.debug('Reading chunked trailer.')
-        trailer_data = yield tornado.gen.Task(self._io_stream.read_until_regex,
-            br'\r?\n\r?\n')
+        trailer_data = yield self._io_stream.read_until_regex(br'\r?\n\r?\n')
 
         self._events.response_data.fire(trailer_data)
         response.fields.parse(trailer_data)
@@ -532,13 +538,17 @@ class Connection(object):
         '''Read the response until the connection closes.'''
         _logger.debug('Reading body until close.')
 
-        def response_callback(data):
+        data_queue = self._io_stream.read_until_close_queue()
+
+        while True:
+            data = yield data_queue.get()
+
+            if data is None:
+                break
+
             self._events.response_data.fire(data)
             response.body.content_file.write(self._decompress_data(data))
 
-        yield tornado.gen.Task(self._io_stream.read_until_close,
-            streaming_callback=response_callback)
-
         response.body.content_file.write(self._flush_decompressor())
 
     @property
@@ -581,7 +591,6 @@ class Connection(object):
 
 class HostConnectionPool(collections.Set):
     '''A Connection pool to a particular server.'''
-    # TODO: remove old connection instances
     def __init__(self, host, port, request_queue=None, ssl=False, max_count=6,
     connection_factory=Connection):
         assert isinstance(host, str)
@@ -595,7 +604,19 @@ class HostConnectionPool(collections.Set):
         self._max_count = max_count
         self._max_count_semaphore = toro.BoundedSemaphore(max_count)
         self._running = True
+        self._cleaner_timer = tornado.ioloop.PeriodicCallback(
+            self.clean, 300000)
         self._run()
+        self._cleaner_timer.start()
+
+    @property
+    def active(self):
+        '''Return whether connections are active or items are queued.'''
+        for connection in self._connections:
+            if connection.active:
+                return True
+
+        return self._request_queue.qsize() > 0
 
     @tornado.gen.coroutine
     def put(self, request, kwargs, async_result):
@@ -625,11 +646,7 @@ class HostConnectionPool(collections.Set):
 
     @tornado.gen.coroutine
     def _process_request(self):
-        try:
-            request, kwargs, async_result = yield self._request_queue.get(
-                deadline=datetime.timedelta(minutes=1))
-        except toro.Timeout:
-            return
+        request, kwargs, async_result = yield self._request_queue.get()
 
         _logger.debug('Host pool got request {0}'.format(request))
 
@@ -644,8 +661,8 @@ class HostConnectionPool(collections.Set):
             async_result.set(error)
         else:
             async_result.set(response)
-        finally:
-            _logger.debug('Host pool done {0}'.format(request))
+
+        _logger.debug('Host pool done {0}'.format(request))
 
     def _get_ready_connection(self):
         _logger.debug('Getting a connection.')
@@ -679,6 +696,7 @@ class HostConnectionPool(collections.Set):
     def stop(self):
         '''Stop the workers.'''
         self._running = False
+        self._cleaner_timer.stop()
 
     def close(self):
         '''Stop workers, close all the connections and remove them.'''
@@ -690,12 +708,14 @@ class HostConnectionPool(collections.Set):
 
         self._connections.clear()
 
-    def clean(self):
-        '''Close and remove connections not in use.'''
+    def clean(self, force_close=False):
+        '''Remove connections not in use.'''
         for connection in tuple(self._connections):
-            if not connection.active:
+            if not connection.active \
+            and (force_close or not connection.connected):
                 connection.close()
                 self._connections.remove(connection)
+                _logger.debug('Cleaned connection {0}'.format(connection))
 
 
 class ConnectionPool(collections.Mapping):
@@ -741,6 +761,18 @@ class ConnectionPool(collections.Mapping):
 
         self._pools.clear()
 
+    def clean(self):
+        '''Remove Host Connection Pools not in use.'''
+        for key in tuple(self._pools.keys()):
+            pool = self._pools[key]
+
+            pool.clean()
+
+            if not pool.active:
+                pool.stop()
+                del self._pools[key]
+                _logger.debug('Cleaned host pool {0}.'.format(pool))
+
 
 class Client(BaseClient):
     '''HTTP/1.1 client.
